@@ -30,6 +30,7 @@
 #include <atk/atkobject.h>
 #include <atk/atknoopobject.h>
 #include <libspi/Accessibility.h>
+#include <libspi/spi-private.h>
 #include "accessible.h"
 #include "application.h"
 
@@ -40,10 +41,9 @@
 #define DBG(a,b) if(_dbg>=(a))b
 
 static int _dbg = 0;
-static char *spi_nil_string = "";
-
 static CORBA_Environment ev;
-static Accessibility_Registry registry = NULL;
+static Accessibility_Registry registry = CORBA_OBJECT_NIL;
+static Accessibility_DeviceEventController device_event_controller = CORBA_OBJECT_NIL;
 static SpiApplication *this_app = NULL;
 static gboolean registry_died = FALSE;
 static gboolean atk_listeners_registered = FALSE;
@@ -62,10 +62,10 @@ static guint toplevel_handler;
 
 static guint atk_signal_text_changed;
 static guint atk_signal_child_changed;
+static guint atk_signal_active_descendant_changed;
 
 /* NOT YET USED
    static guint atk_signal_text_selection_changed;
-   static guint atk_signal_active_descendant_changed;
    static guint atk_signal_row_reordered;
    static guint atk_signal_row_inserted;
    static guint atk_signal_row_deleted;
@@ -128,6 +128,9 @@ spi_atk_bridge_init_event_type_consts ()
 					      ATK_TYPE_OBJECT);
   atk_signal_text_changed = g_signal_lookup ("text_changed", 
 					     ATK_TYPE_TEXT);
+  atk_signal_active_descendant_changed = 
+         g_signal_lookup ("active_descendant_changed", 
+		          ATK_TYPE_OBJECT);
 }
 
 static int
@@ -221,7 +224,7 @@ spi_atk_bridge_register_application (Accessibility_Registry registry)
 }
 
 static Accessibility_Registry
-spi_atk_bridge_get_registry ()
+spi_atk_bridge_get_registry (void)
 {
   CORBA_Environment ev;
 
@@ -248,6 +251,34 @@ spi_atk_bridge_get_registry ()
   return registry;
 }
 
+static Accessibility_DeviceEventController
+spi_atk_bridget_get_dec (void)
+{
+  CORBA_Environment ev;
+
+  if (device_event_controller != CORBA_OBJECT_NIL)
+    {
+      if (ORBit_small_get_connection_status (device_event_controller)
+	  == ORBIT_CONNECTION_CONNECTED)
+        return device_event_controller;
+    }
+
+  CORBA_exception_init (&ev);
+
+  device_event_controller =
+    Accessibility_Registry_getDeviceEventController (
+	    spi_atk_bridge_get_registry (), &ev);
+
+  if (BONOBO_EX (&ev))
+    {
+      g_warning ("failure: no deviceeventcontroller found\n");
+      registry_died = TRUE;
+      device_event_controller = CORBA_OBJECT_NIL;
+    }
+
+  return device_event_controller;
+}
+
 int
 gtk_module_init (gint *argc, gchar **argv[])
 {
@@ -317,6 +348,7 @@ spi_atk_register_event_listeners (void)
 
   add_signal_listener ("Gtk:AtkObject:children-changed");
   add_signal_listener ("Gtk:AtkObject:visible-data-changed");
+  add_signal_listener ("Gtk:AtkObject:active-descendant-changed");
   add_signal_listener ("Gtk:AtkSelection:selection-changed");
   add_signal_listener ("Gtk:AtkText:text-selection-changed");
   add_signal_listener ("Gtk:AtkText:text-changed");
@@ -347,6 +379,7 @@ deregister_application (BonoboObject *app)
   Accessibility_Registry registry = spi_atk_bridge_get_registry ();	
   Accessibility_Registry_deregisterApplication (registry, BONOBO_OBJREF (app), &ev);
 
+  device_event_controller = bonobo_object_release_unref (device_event_controller, &ev);
   registry = bonobo_object_release_unref (registry, &ev);
   
   app = bonobo_object_unref (app);
@@ -437,10 +470,13 @@ spi_atk_bridge_focus_tracker (AtkObject *object)
   e.detail1 = 0;
   e.detail2 = 0;
   spi_init_any_nil (&e.any_data);
+
+  CORBA_exception_init (&ev);
   Accessibility_Registry_notifyEvent (spi_atk_bridge_get_registry (), &e, &ev);
-  if (BONOBO_EX (&ev)) registry_died = TRUE;
-  
-  Accessibility_Accessible_unref (e.source, &ev);
+  if (BONOBO_EX (&ev))
+    registry_died = TRUE;
+
+  bonobo_object_unref (source);
   
   CORBA_exception_free (&ev);
 }
@@ -488,7 +524,7 @@ spi_atk_emit_eventv (const GObject         *gobject,
       e.detail2 = detail2;
       if (any) e.any_data = *any;
       else spi_init_any_nil (&e.any_data);
-      
+
 #ifdef SPI_BRIDGE_DEBUG
       s = Accessibility_Accessible__get_name (BONOBO_OBJREF (source), &ev);
       g_warning ("Emitting event '%s' (%lu, %lu) on %s",
@@ -586,7 +622,6 @@ spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
   return TRUE;
 }
 
-
 static void
 spi_init_keystroke_from_atk_key_event (Accessibility_DeviceEvent  *keystroke,
 				       AtkKeyEventStruct          *event)
@@ -642,90 +677,19 @@ spi_atk_bridge_key_listener (AtkKeyEventStruct *event, gpointer data)
 {
   CORBA_boolean             result;
   Accessibility_DeviceEvent key_event;
-  Accessibility_DeviceEventController controller;
-	
-  if (BONOBO_EX (&ev))
-	g_warning ("failure: pre-listener get dec\n");
 
-  controller =
-    Accessibility_Registry_getDeviceEventController (
-	    spi_atk_bridge_get_registry (), &ev);
+  CORBA_exception_init (&ev);
 
-  if (BONOBO_EX (&ev))
-    {
-      g_warning ("failure: no deviceeventcontroller found\n");
-      CORBA_exception_free (&ev);
-      registry_died = TRUE;
-      result = FALSE;
-    }
-  else
-    {
+  spi_init_keystroke_from_atk_key_event (&key_event, event);
 
-      spi_init_keystroke_from_atk_key_event (&key_event, event);
+  result = Accessibility_DeviceEventController_notifyListenersSync (
+	  spi_atk_bridget_get_dec (), &key_event, &ev);
 
-      result = Accessibility_DeviceEventController_notifyListenersSync (
-        controller, &key_event, &ev);
-
-      bonobo_object_release_unref (controller, &ev);
-      CORBA_exception_free (&ev);
-    }
+  CORBA_exception_free (&ev);
 
   return result;
 }
 
-
-static void
-spi_atk_signal_emit_event (const GObject *gobject, 
-			   const GSignalQuery *signal_query,
-			   long detail1, 
-			   long detail2,
-			   const gchar *name, 
-			   const gchar *detail)
-{
-  CORBA_any any;
-  CORBA_char *sp = NULL;
-  AtkObject *ao;
-
-  if (signal_query->signal_id == atk_signal_text_changed)
-    {
-      sp = atk_text_get_text (ATK_TEXT (gobject),
-			      detail1,
-			      detail1+detail2);
-      spi_init_any_string (&any, &sp);
-    }
-#ifdef EXTENDED_OBJECT_EVENTS_ARE_WORKING
-  else if ((signal_query->signal_id == atk_signal_child_changed) && gobject)
-    {
-      ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
-					    detail1);
-      if (ao) 
-	{
-	  spi_init_any_object (&any, ao); 
-	  atk_object_unref (ao);
-	}
-      else
-	{
-	  spi_init_any_nil (&any);
-	}
-    }
-#endif
-  else
-    {
-      spi_init_any_nil (&any);
-    }
-
-  if (detail)
-    spi_atk_emit_eventv (gobject, detail1, detail2, &any,
-			 "object:%s:%s", name, detail);
-  else
-    spi_atk_emit_eventv (gobject, detail1, detail2, &any,
-			 "object:%s", name);
-  if (sp)
-    g_free (sp);
-}
-
-
-
 static gboolean
 spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
 				guint n_param_values,
@@ -736,9 +700,12 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   GSignalQuery signal_query;
   const gchar *name;
   const gchar *detail;
-  CORBA_char *sp;
-  
+  CORBA_any any;
+  CORBA_Object c_obj;
+  char *sp = NULL;
+  AtkObject *ao;
   gint detail1 = 0, detail2 = 0;
+  SpiAccessible *s_ao = NULL;
 #ifdef SPI_BRIDGE_DEBUG
   const gchar *s, *s2;
 #endif 
@@ -760,14 +727,66 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
 #endif
   
   gobject = g_value_get_object (param_values + 0);
-  if (G_VALUE_TYPE (param_values + 1) == G_TYPE_INT)
-    detail1 = g_value_get_int (param_values + 1);
-  if (G_VALUE_TYPE (param_values + 2) == G_TYPE_INT)
-    detail2 = g_value_get_int (param_values + 2);
-  
-  spi_atk_signal_emit_event (gobject, &signal_query,
-			     detail1, detail2,
-			     name, detail);
+
+  if (signal_query.signal_id == atk_signal_active_descendant_changed)
+    {
+      gpointer child = g_value_get_pointer (param_values + 1);
+
+      g_return_val_if_fail (ATK_IS_OBJECT (child), TRUE);
+
+      ao = ATK_OBJECT (child);
+
+      detail1 = atk_object_get_index_in_parent (ao);
+      s_ao = spi_accessible_new (ao);
+      c_obj = BONOBO_OBJREF (s_ao);
+      spi_init_any_object (&any, &c_obj);
+    }
+  else
+    {
+      if (G_VALUE_TYPE (param_values + 1) == G_TYPE_INT)
+        detail1 = g_value_get_int (param_values + 1);
+      if (G_VALUE_TYPE (param_values + 2) == G_TYPE_INT)
+        detail2 = g_value_get_int (param_values + 2);
+
+      if (signal_query.signal_id == atk_signal_text_changed)
+        {
+          sp = atk_text_get_text (ATK_TEXT (gobject),
+	    		          detail1,
+			          detail1+detail2);
+          spi_init_any_string (&any, &sp);
+        }
+      else if ((signal_query.signal_id == atk_signal_child_changed) && gobject)
+        {
+          ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
+					        detail1);
+          if (ao) 
+	    {
+              s_ao = spi_accessible_new (ao);
+              c_obj = BONOBO_OBJREF (s_ao);
+              spi_init_any_object (&any, &c_obj);
+	      g_object_unref (ao);
+	    }
+          else
+	    {
+	      spi_init_any_nil (&any);
+	    }
+        }
+      else
+        {
+	  spi_init_any_nil (&any);
+        }
+    }
+
+  if (detail)
+    spi_atk_emit_eventv (gobject, detail1, detail2, &any,
+			 "object:%s:%s", name, detail);
+  else
+    spi_atk_emit_eventv (gobject, detail1, detail2, &any,
+			 "object:%s", name);
+
+  if (sp)
+    g_free (sp);
+
   return TRUE;
 }
 
@@ -799,7 +818,7 @@ spi_atk_bridge_window_event_listener (GSignalInvocationHint *signal_hint,
   gobject = g_value_get_object (param_values + 0);
 
   s = atk_object_get_name (ATK_OBJECT (gobject));
-  spi_init_any_string (&any, &s);
+  spi_init_any_string (&any, (char **) &s);
   
   spi_atk_emit_eventv (gobject, 0, 0, &any,
 		       "window:%s", name);
