@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,58 +22,120 @@
  */
 
 #include <stdlib.h>
+#include <unistd.h>
 #include "../cspi/spi-private.h" /* A hack for now */
 
 static void traverse_accessible_tree (Accessible *accessible);
 
 static void report_event  (const AccessibleEvent *event, void *user_data);
 static void report_detail_event  (const AccessibleEvent *event, void *user_data);
+static void report_text_event  (const AccessibleEvent *event, void *user_data);
+static void timing_test_event (const AccessibleEvent *event, void *user_data);
+static SPIBoolean report_mouse_event  (const AccessibleDeviceEvent *event, void *user_data);
 
 static AccessibleEventListener *generic_listener;
 static AccessibleEventListener *specific_listener;
+static AccessibleEventListener *test_listener;
+static AccessibleEventListener *text_listener;
+static AccessibleDeviceListener *mouse_device_listener;
 static gint n_elements_traversed = 0;
+static GTimer *timer;
+
+static gboolean report_mouse_events = TRUE;
+
+void 
+usage_and_exit( void )
+{
+  g_print("\nUsage: event-listener-test [-h] [-m]\n");
+  g_print("       -h    : prints this usage message.\n");
+  g_print("       -m    : disable mouse event reporting.\n\n");
+
+  exit( 1 );
+}
 
 int
 main (int argc, char **argv)
 {
-  int i, j;
+  int i, j, c;
   int n_desktops;
   int n_apps;
   char *s;
-  GTimer *timer;
   gdouble elapsed_time;
   Accessible *desktop;
   Accessible *application;
   const char *modules;
 
+  /* Parse Command-line */
+  if ( argc > 1 ) {
+      while ( ( c = getopt( argc, argv, "hm")) != EOF ) {
+          switch( c ) {
+              case 'm':
+                  report_mouse_events = FALSE;
+                  break;
+              default:
+                  usage_and_exit();
+                  break;
+          }
+      }
+      if ( optind < argc ) {
+          usage_and_exit();
+      }
+  }
+
+  fprintf (stderr, "RUNNING\n");
+
   SPI_init ();
 
   generic_listener = SPI_createAccessibleEventListener (
 	  report_event, NULL); 
   specific_listener = SPI_createAccessibleEventListener (
 	  report_detail_event, NULL); 
+  text_listener = SPI_createAccessibleEventListener (
+	  report_text_event, NULL);
+  test_listener = SPI_createAccessibleEventListener (
+	  timing_test_event, NULL);
+  mouse_device_listener = SPI_createAccessibleDeviceListener (
+          report_mouse_event, NULL);
 
   SPI_registerGlobalEventListener (generic_listener,
 				   "focus:");
+  if ( report_mouse_events ) {
+      SPI_registerGlobalEventListener (specific_listener,
+				       "mouse:rel");
+      SPI_registerGlobalEventListener (specific_listener,
+				       "mouse:button");
+      SPI_registerGlobalEventListener (specific_listener,
+				       "mouse:abs");
+  }
+  SPI_registerDeviceEventListener (mouse_device_listener, 
+				   SPI_BUTTON_PRESSED | SPI_BUTTON_RELEASED,
+				   NULL);
+  SPI_registerGlobalEventListener (specific_listener,
+				   "keyboard:modifiers");
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:property-change");
-  SPI_registerGlobalEventListener (specific_listener,
-				   "object:property-change:accessible-name");
+/*  SPI_registerGlobalEventListener (specific_listener,
+    "object:property-change:accessible-name");*/
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:state-changed"); 
+/*  SPI_registerGlobalEventListener (specific_listener,
+    "object:state-changed:focused"); */
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:selection-changed"); 
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:children-changed"); 
-  SPI_registerGlobalEventListener (specific_listener,
-				   "object:children-changed:add"); 
+/*  SPI_registerGlobalEventListener (specific_listener,
+    "object:children-changed:add"); */
+  SPI_registerGlobalEventListener (generic_listener,
+				   "object:active-descendant"); 
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:visible-data-changed"); 
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:text-selection-changed"); 
+
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:text-caret-moved"); 
-  SPI_registerGlobalEventListener (generic_listener,
+  SPI_registerGlobalEventListener (text_listener,
 				   "object:text-changed"); 
   SPI_registerGlobalEventListener (generic_listener,
 				   "object:column-inserted"); 
@@ -100,7 +163,6 @@ main (int argc, char **argv)
 	                           "window:deactivate");
   SPI_registerGlobalEventListener (generic_listener,
 	                           "window:close");
-#ifdef NOT_YET_IMPLEMENTED
   SPI_registerGlobalEventListener (generic_listener,
 	                           "window:lower");
   SPI_registerGlobalEventListener (generic_listener,
@@ -111,6 +173,9 @@ main (int argc, char **argv)
 	                           "window:shade");
   SPI_registerGlobalEventListener (generic_listener,
 	                           "window:unshade");
+  SPI_registerGlobalEventListener (test_listener,
+				   "object:test");
+#ifdef NOT_YET_IMPLEMENTED
   /* event below possibly should just be property change? */
   SPI_registerGlobalEventListener (generic_listener,
 	                           "window:restyle"); 
@@ -132,7 +197,7 @@ main (int argc, char **argv)
   g_print ("[%f elements/sec, %f SPI calls/sec]\n", 
 	n_elements_traversed/g_timer_elapsed(timer, NULL),
 	(n_elements_traversed*8+1)/g_timer_elapsed(timer, NULL));
-  
+  g_timer_reset (timer);
   SPI_event_main ();
 
   putenv ("AT_BRIDGE_SHUTDOWN=1");
@@ -178,13 +243,64 @@ traverse_accessible_tree (Accessible *accessible)
 void
 report_event (const AccessibleEvent *event, void *user_data)
 {
-  fprintf (stderr, "%s\n", event->type);
+  static long count = 0;
+  char *s = Accessible_getName (event->source);
+  fprintf (stderr, "%s %s\n", event->type, s);
+  if (s) SPI_freeString (s);
+  if (count == 0) {
+	  g_timer_reset (timer);
+	  g_timer_start (timer);
+  }
+  ++count;
+  if ((count % 100) == 0) {
+	  g_print ("%d events received, %f events/sec\n",
+		   count,
+		   count/g_timer_elapsed(timer, NULL));
+  }
 }
 
 void
 report_detail_event (const AccessibleEvent *event, void *user_data)
 {
-  fprintf (stderr, "%s\n", event->type);
+  char *s = Accessible_getName (event->source);
+  fprintf (stderr, "(detail) %s %s %d %d\n", event->type, s,
+	   event->detail1, event->detail2);
+  if (s) SPI_freeString (s);
+}
+
+void
+report_text_event (const AccessibleEvent *event, void *user_data)
+{
+  char *s = Accessible_getName (event->source);
+  fprintf (stderr, "(detail) %s %s %d %d\n", event->type, s,
+	   event->detail1, event->detail2);
+  if (s) SPI_freeString (s);
+  s = AccessibleTextChangedEvent_getChangeString (event);
+  fprintf (stderr, "context string %s\n", (s) ? s : "<nil>");
+}
+
+SPIBoolean
+report_mouse_event (const AccessibleDeviceEvent *event, void *user_data)
+{
+  fprintf (stderr, "mouse event %ld %d %x %x\n", 
+	   event->keyID, 
+	   (int) event->keycode,
+	   (unsigned) event->type,
+	   (unsigned) event->modifiers);
+  return FALSE;
+}
+
+void
+timing_test_event (const AccessibleEvent *event, void *user_data)
+{
+	static long count = 0;
+	if (count == 0) g_timer_start (timer);
+	++count;
+	if ((count % 500) == 0) {
+		g_print ("%d events received, %f events/sec\n",
+			 count,
+			 count/g_timer_elapsed(timer, NULL));
+	}
 }
 
 void
